unit plxControls;

interface

uses Windows, Messages, SysUtils, Forms, Classes, Grids, ComCtrls, Controls, StdCtrls;

type
  TSelectType = (stSelNone, stSelAll, stSelInvert, stSelSpecial);
  TLVDelType = (dtSelected, dtUnSelected, dtChecked, dtUnChecked, dtAll);

procedure CopyGrid(FromGrid, ToGrid: TStringGrid);
procedure SaveLVToFile(LV: TListView; FName: string);
procedure LoadLVFromFile(LV: TListView; FName: string);
procedure RemoveDoubledListItems(AListItems: TListItems);
procedure SelectLVItems(LView: TListView; SelType: TSelectType);
procedure RemoveLVItems(LView: TListView; DelType: TLVDelType);
procedure ClearAllEdits(ParentControl: TWinControl);
procedure SetAllEdits(ParentControl: TWinControl; TextToSet: String);
procedure SetFormTopmost(Frm: TForm; OnTop: Boolean);
procedure MoveControl(Cntrl: TControl);


implementation

procedure CopyGrid(FromGrid, ToGrid: TStringGrid);
var I, J: Integer;
begin
  ToGrid.RowCount := FromGrid.RowCount;
  ToGrid.ColCount := FromGrid.ColCount;
  for I := 0 to FromGrid.RowCount - 1 do
    for J := 0 to FromGrid.ColCount - 1 do
      ToGrid.Cells[J, I] := FromGrid.Cells[J, I];
end;

procedure SaveLVToFile(LV: TListView; FName: string);
var T: TextFile;
    I, J: Integer;
begin
  AssignFile(T, FName);
  Rewrite(T);
  WriteLn(T, IntToStr(LV.Items.Count));
  WriteLn(T, IntToStr(LV.Columns.Count));
  for I := 0 to LV.Items.Count - 1 do
    with LV.Items[I] do begin
      WriteLn(T, Caption);
      for J := 0 to SubItems.Count - 1 do WriteLn(T, SubItems[J]);
    end;
  CloseFile(T);
end;

procedure LoadLVFromFile(LV: TListView; FName: string);
var T: TextFile;
    S: string;
    I, J: Integer;
    ItemsCount, ColumnsCount: Integer;
begin
  AssignFile(T, FName);
  Reset(T);
  ReadLn(T, S);
  ItemsCount := StrToInt(S);
  ReadLn(T, S);
  ColumnsCount := StrToInt(S);
  LV.Items.Clear;
  for I := 0 to ItemsCount - 1 do
    with LV.Items.Add do begin
      ReadLn(T, S);
      Caption := S;
      for J := 0 to ColumnsCount - 2 do begin
        ReadLn(T, S);
        SubItems.Add(S);
      end;
    end;
  CloseFile(T);
end;

procedure RemoveDoubledListItems(AListItems: TListItems);
//sss da chisti i po subitems
var StubStrings: TStrings;
  I: Integer;
begin
  StubStrings := TStringList.Create;
  I := 0;
  AListItems.BeginUpdate;
  while I < AListItems.Count do
    if StubStrings.IndexOf(AListItems[I].Caption) = -1 then begin
      StubStrings.Add(AListItems[I].Caption);
      Inc(I);
    end
    else AListItems.Delete(I);
  AListItems.EndUpdate;
  StubStrings.Free;
end;

procedure SelectLVItems(LView: TListView; SelType: TSelectType);
var I: Integer;
begin
  with LView do
    for I := 0 to Items.Count - 1 do
      case SelType of
        stSelNone: Items[I].Selected := False;
        stSelAll: Items[I].Selected := True;
        stSelInvert: Items[I].Selected := not Items[I].Selected;
      end;
end;

procedure RemoveLVItems(LView: TListView; DelType: TLVDelType);
var I: Integer;
begin
  if LView.Items.Count = 0 then Exit;
  I := 0;
  while I < LView.Items.Count do with LView.Items do
  case DelType of
    dtSelected: if LView.Items[I].Selected then LView.Items.Delete(I)
                  else Inc(I);
    dtUnSelected: if not LView.Items[I].Selected then LView.Items.Delete(I)
                  else Inc(I);
    dtChecked: if LView.Items[I].Checked then LView.Items.Delete(I)
                  else Inc(I);
    dtUnChecked: if not LView.Items[I].Checked then LView.Items.Delete(I)
                  else Inc(I);
    dtAll: LView.Items.Delete(I);
  end;
end;

procedure ClearAllEdits(ParentControl: TWinControl);
begin
  SetAllEdits(ParentControl, '');
end;

procedure SetAllEdits(ParentControl: TWinControl; TextToSet: String);
var I: Integer;
begin
  for I := 0 to ParentControl.ControlCount - 1 do
    if ParentControl.Controls[I] is TEdit then
      TEdit(ParentControl.Controls[I]).Text := TextToSet;
end;

procedure SetFormTopmost(Frm: TForm; OnTop: Boolean);
begin
  if OnTop then SetWindowPos(Frm.Handle, hwnd_TopMost, 0, 0, 0, 0, swp_NoMove + swp_NoSize)
  else SetWindowPos(Frm.Handle, hwnd_NoTopMost, 0, 0, 0, 0, swp_NoMove + swp_NoSize);
end;

procedure MoveControl(Cntrl: TControl);
begin
  Releasecapture;
  Cntrl.Perform(WM_syscommand, $F012, 0);
end;


end.
