unit plxGraph;

interface
uses Classes, Windows, SysUtils, Graphics, Controls, Registry, ShellAPI, ImgList,
     ExtCtrls;


function GetDefaultIconsEx(IcoType: Integer): HIcon;
function GetDefaultIconLarge: HIcon;
function GetDefaultIconSmall: HIcon;
function GetTypeIconsEx(FName: string; LargeIco: Boolean): HIcon;
function GetTypeIconLarge(FName: string): HIcon;
function GetTypeIconSmall(FName: string): HIcon;
function GetFileIconEx(FName: string; LargeIco: Boolean): TIcon;
function GetFileIconSmall(FName: string): TIcon;
function GetFileIconLarge(FName: string): TIcon;
function CreatFileTypeThumb(FName: string; ThumbSize: Integer; Is3D: Boolean): TBitmap;
procedure CreateThumbFrame(VBitmap: TBitmap; ThmbWidth, ThmbHeight: Integer; Is3D: Boolean);
function CreateDragImage(DragContr: TWinControl): TBitmap;


implementation

const LargeDefaultIcon = 0;
      SmallDefaultIcon = 1;

function GetDefaultIconsEx(IcoType: Integer): HIcon;
var Reg: TRegistry;
  HandleLarge, HandleSmall: array[1..5] of HIcon;
  ShellFile: string;
begin
  Result := 0;
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  Reg.OpenKey('\SOFTWARE\Microsoft', False);
  if Reg.KeyExists('Windows NT') then begin
    Reg.OpenKey('Windows NT\CurrentVersion', False);
    ShellFile := '\system32\shell32.dll';
  end
  else begin
    Reg.OpenKey('Windows\CurrentVersion', False);
    ShellFile := '\system\shell32.dll';
  end;
  ShellFile := Reg.ReadString('SystemRoot') + ShellFile;
  Reg.Free;
  ExtractIconEx(PChar(ShellFile), 0, HandleLarge[1], HandleSmall[1], 5);
  case IcoType of
    LargeDefaultIcon: Result := HandleLarge[1];
    SmallDefaultIcon: Result := HandleSmall[1];
  end;
end;

function GetDefaultIconLarge: HIcon;
begin
  Result := GetDefaultIconsEx(LargeDefaultIcon);
end;

function GetDefaultIconSmall: HIcon;
begin
  Result := GetDefaultIconsEx(SmallDefaultIcon);
end;

function GetTypeIconsEx(FName: string; LargeIco: Boolean): HIcon;
var Handlelarge, Handlesmall: HIcon;
  StubStr, FileExtension, IconFileName: string;
  CommaPos, IconIndex: Integer;
begin
  Result := 0;
  FileExtension := ExtractFileExt(FName);
  try
    with TRegistry.Create do begin
      RootKey := HKEY_CLASSES_ROOT;
      OpenKey(FileExtension, False);
      StubStr := ReadString('');
      OpenKey('\' + StubStr, False);
      StubStr := ReadString('');
      OpenKey('DefaultIcon', False);
      IconFileName := ReadString('');
      CommaPos := Pos(',', IconFileName);
      IconIndex := StrtoIntDef(Copy(IconFileName, CommaPos + 1, Length(IconFileName) - CommaPos), -1);
      IconFileName := Copy(IconFileName, 1, CommaPos - 1);
      if IconFileName = '' then
        if LargeIco then HandleLarge := GetDefaultIconsEx(LargeDefaultIcon)
        else HandleSmall := GetDefaultIconsEx(SmallDefaultIcon)
      else ExtractIconEx(Pchar(IconFileName), IconIndex, HandleLarge, HandleSmall, 1);
      if LargeIco then Result := HandleLarge
      else Result := HandleSmall;
      Free;
    end;
  except
  end;
end;

function GetTypeIconLarge(FName: string): HIcon;
begin
  Result := GetTypeIconsEx(FName, True);
end;

function GetTypeIconSmall(FName: string): HIcon;
begin
  Result := GetTypeIconsEx(FName, False);
end;

function GetFileIconEx(FName: string; LargeIco: Boolean): TIcon;
var StubIco: TIcon;
  StubImages: TImageList;
  FileInfo: SHFileInfo;
begin
  StubIco := TIcon.Create;
  StubImages := TImageList.Create(nil);
  StubImages.ShareImages := True;
  StubImages.Masked := True;
  StubImages.bkColor := clNone;
  StubImages.BlendColor := clNone;
  StubImages.DrawingStyle := dsNormal;
  if LargeIco then StubImages.Handle := SHGetFileInfo(PChar(FName), 0, FileInfo, SizeOf(FileInfo), SHGFI_SYSICONINDEX or SHGFI_LARGEICON)
  else StubImages.Handle := SHGetFileInfo(PChar(FName), 0, FileInfo, SizeOf(FileInfo), SHGFI_SYSICONINDEX or SHGFI_SMALLICON);
  StubImages.GetIcon(FileInfo.iIcon, StubIco);
  Result := StubIco;
  StubImages.Free;
end;

function GetFileIconSmall(FName: string): TIcon;
begin
  Result := GetFileIconEx(FName, False);
end;

function GetFileIconLarge(FName: string): TIcon;
begin
  Result := GetFileIconEx(FName, True);
end;

function CreatFileTypeThumb(FName: string; ThumbSize: Integer; Is3D: Boolean): TBitmap;
var StubPicture: TPicture;
  StubBitmap: TBitmap;
  IcoPos: Integer;
begin
  StubPicture := TPicture.Create;
  StubBitmap := TBitmap.Create;
  CreateThumbFrame(StubBitmap, ThumbSize, ThumbSize, Is3D);
  StubPicture.Icon.Handle := GetTypeIconLarge(FName);
  IcoPos := (ThumbSize - 32) shr 1;
  StubBitmap.Canvas.Draw(IcoPos, IcoPos, StubPicture.Graphic);
  Result := StubBitmap;
  StubPicture.Free;
end;

procedure CreateThumbFrame(VBitmap: TBitmap; ThmbWidth, ThmbHeight: Integer; Is3D: Boolean);
var FrameRect: TRect;
begin
  VBitmap.Width := ThmbWidth;
  VBitmap.Height := ThmbHeight;
  VBitmap.Canvas.Brush.Color := clWhite;
  if not Is3D then VBitmap.Canvas.Pen.Width := 0;
  FrameRect := Rect(0, 0, ThmbWidth, ThmbHeight);
  VBitmap.Canvas.Rectangle(FrameRect);
  if Is3D then begin
    Frame3D(VBitmap.Canvas, FrameRect, clBtnface, clBlack, 1);
    Frame3D(VBitmap.Canvas, FrameRect, clBtnHighlight, clBtnShadow, 1);
  end;
end;

function CreateDragImage(DragContr: TWinControl): TBitmap;
var DC: HDC;
  BM: TBitmap;
  W, H: longint;
begin
  with DragContr do begin
    BM := TBitmap.Create;
    DC := GetDCEx(handle, 0, DCX_PARENTCLIP);
    W := Width;
    H := Height;
    BM.Width := W;
    BM.Height := H;
    StretchBlt(BM.canvas.handle, 0, 0, W, H, DC, 0, 0, W, H, SRCCOPY);
    ReleaseDC(Handle, DC);
  end;
  Result := BM;
end;



end.
