unit plxRegistry;

interface

uses SysUtils, Windows, Classes, Registry, ComCtrls, Forms, Grids, Graphics,
     StdCtrls, Controls;

function RegKeyExists(RegKeyStr: string): Boolean;
function BoolToReg(RegKeyStr, ValueName: string; BoolToStore: Boolean): Boolean;
function RegToBool(RegKeyStr, ValueName: string): Boolean;
function IntToReg(RegKeyStr, ValueName: string; IntToStore: Integer): Boolean;
function RegToInt(RegKeyStr, ValueName: string): Integer;
function StrToReg(RegKeyStr, ValueName, StrToStore: string): Boolean;
function RegToStr(RegKeyStr, ValueName: string): string;

function StringsToReg(RegKeyStr, ValueName: string; StringsToStore: TStrings): Boolean;
function RegToStrings(RegKeyStr, ValueName: string): TStrings;
function ColorToReg(RegKeyStr, ValueName: string; ColorToStore: TColor): Boolean;
function RegToColor(RegKeyStr, ValueName: string): TColor;

function RegToListView(RegKeyStr, ValueName: string; LView: TListView): Boolean;
function ListViewToReg(RegKeyStr, ValueName: string; LView: TListView): Boolean;
function GridSizeToStr(GridToConvert: TStringGrid): string;
procedure StrToGridSize(StringToConvert: string; var GridToSet: TStringGrid);
function StringGridToReg(RegKeyStr, ValueName: string; StringGridToStore: TStringGrid): Boolean;
procedure RegToStringGrid(RegKeyStr, ValueName: string; StringGridToRestore: TStringGrid);
function FormSizeToStr(FormToConvert: TForm): string;
procedure StrToFormSize(StrToConvert: string; FormToSet: TForm);
function FormToReg(RegKeyStr, ValueName: string; FormToStore: TForm): Boolean;
procedure RegToForm(RegKeyStr, ValueName: string; FormToRestore: TForm);
function GetRegSubTree(MainKey: LongInt; var aList :TStringList; aKey: string): Boolean;

function EditsToReg(ParentComp: TComponent; RegKeyStr: String): Boolean;
procedure RegToEdits(ParentComp: TComponent; RegKeyStr: String);
function ChecksToReg(ParentComp: TComponent; RegKeyStr: String): Boolean;
procedure RegToChecks(ParentComp: TComponent; RegKeyStr: String);
function ComboToReg(RegKeyStr: String; ACombo: TComboBox): Boolean;
procedure RegToCombo(RegKeyStr: String; ACombo: TComboBox);

const ItemDelimiter = #8;

implementation
uses plxStrings;

function RegKeyExists(RegKeyStr: string): Boolean;
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  Result := Reg.KeyExists(RegKeyStr);
  Reg.Free;
end;

function BoolToReg(RegKeyStr, ValueName: string; BoolToStore: Boolean): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteBool(ValueName, BoolToStore);
    Free;
  end
end;

function RegToBool(RegKeyStr, ValueName: string): Boolean;
begin
  Result := False;
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then Result := ReadBool(ValueName);
    Free;
  end;
end;

function IntToReg(RegKeyStr, ValueName: string; IntToStore: Integer): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteInteger(ValueName, IntToStore);
    Free;
  end
end;

function RegToInt(RegKeyStr, ValueName: string): Integer;
begin
  Result := -1;
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then Result := ReadInteger(ValueName);
    Free;
  end;
end;

function StrToReg(RegKeyStr, ValueName, StrToStore: string): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteString(ValueName, StrToStore);
    Free;
  end
end;

function RegToStr(RegKeyStr, ValueName: string): string;
begin
  Result := '';
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then Result := ReadString(ValueName);
    Free;
  end;
end;

function StringsToReg(RegKeyStr, ValueName: string; StringsToStore: TStrings): Boolean;
var I: Integer;
    StubStr: string;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if not Result then begin
      Free;
      Exit;
    end;
    StubStr := '';
    for I := 0 to StringsToStore.Count - 1 do
      StubStr := StubStr + ItemDelimiter + StringsToStore[I];
    WriteString(ValueName, StubStr);
    Free;
  end;
end;

function RegToStrings(RegKeyStr, ValueName: string): TStrings;
var StubStr: string;
begin
  StubStr := ItemDelimiter;
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then StubStr := ReadString(ValueName);
      Free;
    end;
  Result := ParseString(StubStr, ItemDelimiter, False, False);
end;

function ColorToReg(RegKeyStr, ValueName: string; ColorToStore: TColor): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteString(ValueName, ColorToString(ColorToStore));
    Free;
  end
end;

function RegToColor(RegKeyStr, ValueName: string): TColor;
begin
  Result := clDefault;
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then
          Result := StringToColor(ReadString(ValueName)); ;
      Free;
    end;
end;

function RegToListView(RegKeyStr, ValueName: string; LView: TListView): Boolean;
var StubStr: string;
    ColIndex: Integer;
begin
  Result := False;
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then begin
          StubStr := ReadString(ValueName);
          ColIndex := 0;
          while Pos(ItemDelimiter, StubStr) <> 0 do begin
            LView.Columns[ColIndex].Width := StrToInt(Copy(StubStr, 1, Pos(ItemDelimiter, StubStr) - 1));
            StubStr := Copy(StubStr, Pos(ItemDelimiter, StubStr) + 1, Length(StubStr) - Pos(ItemDelimiter, StubStr));
            Inc(ColIndex);
          end;
          Result := True;
        end;
    Free;
  end;
end;

function ListViewToReg(RegKeyStr, ValueName: string; LView: TListView): Boolean;
var StubStr: string;
    ColIndex: Integer;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if not Result then begin
      Free;
      Exit;
    end;
    StubStr := '';
    for ColIndex := 0 to LView.Columns.Count - 1 do
      StubStr := StubStr + IntToStr(LView.Columns[ColIndex].Width) + ItemDelimiter;
    WriteString(ValueName, StubStr);
    Free;
  end;
end;

function GridSizeToStr(GridToConvert: TStringGrid): string;
var ColIndex: Integer;
    StubStr: string;
begin
  StubStr := '';
  for ColIndex := 0 to GridToConvert.ColCount - 1 do
    StubStr := StubStr + AlignRightStr(IntToStr(GridToConvert.ColWidths[ColIndex]), 4);
  Result := StubStr;
end;

procedure StrToGridSize(StringToConvert: string; var GridToSet: TStringGrid);
var ColIndex: integer;
begin
  for ColIndex := 0 to GridToSet.ColCount - 1 do
    GridToSet.ColWidths[ColIndex] := StrToInt(Copy(StringToConvert, 1+4*ColIndex, 4));
end;

function StringGridToReg(RegKeyStr, ValueName: string; StringGridToStore: TStringGrid): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteString(ValueName, GridSizeToStr(StringGridToStore));
    Free;
  end
end;

procedure RegToStringGrid(RegKeyStr, ValueName: string; StringGridToRestore: TStringGrid);
begin
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then
          StrToGridSize(ReadString(ValueName), StringGridToRestore);
    Free;
  end
end;

function FormSizeToStr(FormToConvert: TForm): string;
var StubStr: string;
begin
  StubStr := AlignRightStr(IntToStr(FormToConvert.Top), 4) + AlignRightStr(IntToStr(FormToConvert.Left), 4) +
    AlignRightStr(IntToStr(FormToConvert.Height), 4) + AlignRightStr(IntToStr(FormToConvert.Width), 4);
  StubStr := StubStr + IntToStr(Integer(FormToConvert.WindowState = wsMaximized));
  Result := StubStr;
end;

procedure StrToFormSize(StrToConvert: string; FormToSet: TForm);
begin
  FormToSet.Top := StrToInt(Copy(StrToConvert, 1, 4));
  FormToSet.Left := StrToInt(Copy(StrToConvert, 5, 4));
  FormToSet.Height := StrToInt(Copy(StrToConvert, 9, 4));
  FormToSet.Width := StrToInt(Copy(StrToConvert, 13, 4));
  if Copy(StrToConvert, 17, 1) = '1' then Application.Minimize; //sss
end;

function FormToReg(RegKeyStr, ValueName: string; FormToStore: TForm): Boolean;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then WriteString(ValueName, FormSizeToStr(FormToStore));
    Free;
  end
end;

procedure RegToForm(RegKeyStr, ValueName: string; FormToRestore: TForm);
begin
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        if ValueExists(ValueName) then
          StrToFormSize(ReadString(ValueName), FormToRestore);
    Free;
  end;
end;

function GetRegSubTree(MainKey: LongInt; var aList :TStringList; aKey: string): Boolean;
var hRoot: HKEY;
    lItem: LongInt;
    hError: LongInt;
    szKey,pData: PChar;
begin
  GetRegSubTree := False;
  if aList = nil then Exit;
  szKey := StrAlloc(Length(aKey) + 1);
  StrPCopy(szKey, aKey);
  lItem := 0;
  pData := StrAlloc(1024);

  hError := RegOpenKey(MainKey, szKey, hRoot);
  if hError = ERROR_SUCCESS then
  begin
     while (hError = ERROR_SUCCESS) do
     begin
        hError := RegEnumKey(hRoot, lItem, pData, 1024);
        if (hError = ERROR_SUCCESS) then
        begin
           GetRegSubTree:=true;
           aList.Add(StrPas(pData));
           Inc(lItem);
        end;
     end;
     RegCloseKey(hRoot);
  end;
  StrDispose(szKey);
  StrDispose(pData);
end;

function EditsToReg(ParentComp: TComponent; RegKeyStr: String): Boolean;
var I: Integer;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then
      for I := 0 to ParentComp.ComponentCount-1 do
        if ParentComp.Components[I] is TEdit then
          with TEdit(ParentComp.Components[I]) do WriteString(Name, Text);
    Free;
  end
end;

procedure RegToEdits(ParentComp: TComponent; RegKeyStr: String);
var I: Integer;
begin
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        for I := 0 to ParentComp.ComponentCount-1 do
          if ParentComp.Components[I] is TEdit then
            with TEdit(ParentComp.Components[I]) do
              if ValueExists(Name) then Text := ReadString(Name);
    Free;
  end;
end;


function ChecksToReg(ParentComp: TComponent; RegKeyStr: String): Boolean;
var I: Integer;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then
      for I := 0 to ParentComp.ComponentCount-1 do
        if ParentComp.Components[I] is TCheckBox then
          with TCheckBox(ParentComp.Components[I]) do WriteBool(Name, Checked);
    Free;
  end
end;

procedure RegToChecks(ParentComp: TComponent; RegKeyStr: String);
var I: Integer;
begin
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then
        for I := 0 to ParentComp.ComponentCount-1 do
          if ParentComp.Components[I] is TCheckBox then
            with TCheckBox(ParentComp.Components[I]) do
              if ValueExists(Name) then Checked := ReadBool(Name);
    Free;
  end;
end;

function ComboToReg(RegKeyStr: String; ACombo: TComboBox): Boolean;
var I: Integer;
begin
  with TRegistry.Create do begin
    Result := OpenKey(RegKeyStr, True);
    if Result then begin
      WriteString(ACombo.Name+'Items', MergeStrings(ACombo.Items, ItemDelimiter));
      WriteString(ACombo.Name+'Text', ACombo.Text);
    end;
    Free;
  end
end;

procedure RegToCombo(RegKeyStr: String; ACombo: TComboBox);
var I: Integer;
begin
  with TRegistry.Create do begin
    if KeyExists(RegKeyStr) then
      if OpenKey(RegKeyStr, False) then begin
        ACombo.Items.Clear;
        if ValueExists(ACombo.Name+'Items') then ACombo.Items := ParseString(ReadString(ACombo.Name+'Items'), ItemDelimiter);
        if ValueExists(ACombo.Name+'Text') then ACombo.Text := ReadString(ACombo.Name+'Text');
      end;
    Free;
  end;
end;


end.
