unit plxStrings;

interface

uses SysUtils, Classes, Graphics;

type
  TRemoveType = (rtAll, rtFirst, rtLast, rtNumber);
  TFindType = (ftFirst, ftLast, ftNumber);

function StrExists(StrToFind, BaseString: string; CaseSens: Boolean = False): boolean;
function RemoveStr(BaseStr, StrToRemove: string;
         RemoveType: TRemoveType = rtAll; RemoveNumber: Integer = 1): string;
function ReplaceStr(BaseStr, StrToReplace, StrToPut: string;
         RemoveType: TRemoveType = rtAll; RemoveNumber: Integer = 1): string;
function StrBefore(BaseStr, FindStr: string; IncludeFindStr: Boolean = False;
                   FindType: TFindType = ftFirst; FindNumber: Integer = 1): string;
function StrAfter(BaseStr, FindStr: string; IncludeFindStr: Boolean = False;
                  FindType: TFindType = ftFirst; FindNumber: Integer = 1): string;
function ParseString(BaseString, BreakString: string; CaseSens: Boolean = False; AllowEmpty: Boolean = True): TStrings;
function MergeStrings(StringsToMerge: TStrings; Delimiter: string): string;
function AlignRightStr(BaseString: string; Positions: Integer): string;
function NormalizeDir(DirStr: string): string;
function UnNormalizeDir(DirStr: string): string;
function BoolToStr(Value: Boolean): string;
function StrToBool(Value: string): Boolean;
function RealToStr(RealNumber: Real): string;
function StrToReal(RealStr: string): Real;
procedure StringToFont(FontStr: string; Font: TFont);
function FontToString(Font: TFont): string;
procedure RemoveDoubledStrings(AStrings: TStrings);
function ColorToHexStr(AColor: TColor): string;
function AddDelString(AStrings: TStrings; StrToAddDel: string): Integer;
function LeadingZero(IntNumber, TotalCount: Integer): string;

function MatchStrings(source, pattern: String): Boolean;

implementation

function StrExists(StrToFind, BaseString: string; CaseSens: Boolean = False): Boolean;
begin
  if CaseSens then Result := Pos(LowerCase(StrToFind), LowerCase(BaseString)) <> 0
    else Result := Pos(StrToFind, BaseString) <> 0;
end;

function RemoveStr(BaseStr, StrToRemove: string;
         RemoveType: TRemoveType = rtAll; RemoveNumber: Integer = 1): string;
var StrPos, I, StrAppearances: Integer;
    ResStr: string;
begin
  ResStr := BaseStr;
  StrPos := Pos(StrToRemove, ResStr);
  if StrPos = 0 then begin
    Result := ResStr;
    Exit;
  end;
  if (RemoveType = rtFirst) or (RemoveType = rtAll) then
    while StrPos <> 0 do begin
      ResStr := Copy(ResStr, 1, StrPos-1) +
                Copy(ResStr, StrPos + Length(StrToRemove), Length(ResStr)-StrPos-Length(StrToRemove)+1);
      if (RemoveType = rtFirst) then Break;
      StrPos := Pos(StrToRemove, ResStr);
    end;
  StrAppearances := 0;
  if (RemoveType = rtNumber) or (RemoveType = rtLast) then
    for I := 0 to Length(ResStr) do begin
      if Copy(ResStr, I, Length(StrToRemove)) = StrToRemove then begin
        Inc(StrAppearances);
        StrPos := I;
      end;
      if (RemoveType = rtNumber) and (StrAppearances = RemoveNumber) then begin
        ResStr := Copy(ResStr, 1, StrPos-1) +
                  Copy(ResStr, StrPos + Length(StrToRemove), Length(ResStr)-StrPos-Length(StrToRemove)+1);
        Break;
      end;
    end;
  if (RemoveType = rtLast) then
    ResStr := Copy(ResStr, 1, StrPos-1) +
              Copy(ResStr, StrPos + Length(StrToRemove), Length(ResStr)-StrPos-Length(StrToRemove)+1);
  Result := ResStr;
end;

function ReplaceStr(BaseStr, StrToReplace, StrToPut: string;
         RemoveType: TRemoveType = rtAll; RemoveNumber: Integer = 1): string;
var StrPos, I, StrAppearances: Integer;
    ResStr: string;
begin
  ResStr := BaseStr;
  StrPos := Pos(StrToReplace, ResStr);
  if StrPos = 0 then begin
    Result := ResStr;
    Exit;
  end;
  if (RemoveType = rtFirst) or (RemoveType = rtAll) then
    while StrPos <> 0 do begin
      ResStr := Copy(ResStr, 1, StrPos-1) + StrToPut +
                Copy(ResStr, StrPos + Length(StrToReplace), Length(ResStr)-StrPos-Length(StrToReplace)+1);
      if (RemoveType = rtFirst) then Break;
      StrPos := Pos(StrToReplace, ResStr);
    end;
  StrAppearances := 0;
  if (RemoveType = rtNumber) or (RemoveType = rtLast) then
    for I := 0 to Length(ResStr) do begin
      if Copy(ResStr, I, Length(StrToReplace)) = StrToReplace then begin
        Inc(StrAppearances);
        StrPos := I;
      end;
      if (RemoveType = rtNumber) and (StrAppearances = RemoveNumber) then begin
        ResStr := Copy(ResStr, 1, StrPos-1) + StrToPut +
                  Copy(ResStr, StrPos + Length(StrToReplace), Length(ResStr)-StrPos-Length(StrToReplace)+1);
        Break;
      end;
    end;
  if (RemoveType = rtLast) then
    ResStr := Copy(ResStr, 1, StrPos-1) + StrToPut +
              Copy(ResStr, StrPos + Length(StrToReplace), Length(ResStr)-StrPos-Length(StrToReplace)+1);
  Result := ResStr;
end;

function StrBefore(BaseStr, FindStr: string; IncludeFindStr: Boolean = False;
                   FindType: TFindType = ftFirst; FindNumber: Integer = 1): string;
var I, MemPos, FoundAppearances: Integer;
begin
  MemPos := 0; Result := ''; FoundAppearances := 0;
  for I := 1 to Length(BaseStr) do
    if Copy(BaseStr, I, Length(FindStr)) = FindStr then begin
      MemPos := I; Inc(FoundAppearances);
      if FindType = ftFirst then Break;
      if (FindType = ftNumber) and (FindNumber = FoundAppearances) then Break;
    end;
  if MemPos > 0 then
    if IncludeFindStr then Result := Copy(BaseStr, 1, MemPos+Length(FindStr)-1)
      else Result := Copy(BaseStr, 1, MemPos-1)
  else;
end;

function StrAfter(BaseStr, FindStr: string; IncludeFindStr: Boolean = False;
                  FindType: TFindType = ftFirst; FindNumber: Integer = 1): string;
var I, MemPos, FoundAppearances: Integer;
begin
  MemPos := 0; Result := ''; FoundAppearances := 0;
  for I := 1 to Length(BaseStr) do
    if Copy(BaseStr, I, Length(FindStr)) = FindStr then begin
      MemPos := I; Inc(FoundAppearances);
      if FindType = ftFirst then Break;
      if (FindType = ftNumber) and (FindNumber = FoundAppearances) then Break;
    end;
  if MemPos > 0 then
    if IncludeFindStr then Result := Copy(BaseStr, MemPos, Length(BaseStr)-MemPos+1)
      else Result := Copy(BaseStr, MemPos+Length(FindStr), Length(BaseStr)-MemPos-Length(FindStr)+1)
  else;
end;

function ParseString(BaseString, BreakString: string; CaseSens: Boolean = False; AllowEmpty: Boolean = True): TStrings;
var EndOfCurrentString: Longint;
    TempStr, TempBaseString: string;
begin
  Result := TStringList.Create;
  TempBaseString := BaseString;
  repeat
    if CaseSens then EndOfCurrentString := Pos(BreakString, TempBaseString)
      else EndOfCurrentString := Pos(LowerCase(BreakString), LowerCase(TempBaseString));
    if EndOfCurrentString = 0 then TempStr := TempBaseString
      else TempStr := Copy(TempBaseString, 1, EndOfCurrentString - 1);
    if ((TempStr = '') and AllowEmpty) or (TempStr <> '') then Result.Add(TempStr);
    TempBaseString := Copy(TempBaseString, EndOfCurrentString + Length(BreakString),
                      Length(TempBaseString) - EndOfCurrentString);
  until EndOfCurrentString = 0;
end;

function MergeStrings(StringsToMerge: TStrings; Delimiter: string): string;
var I : Integer;
begin
  for I := 0 to StringsToMerge.Count-1 do
    Result := Result + StringsToMerge[I] + Delimiter;
  Result := Copy(Result, 1, Length(Result)-Length(Delimiter));
end;

function AlignRightStr(BaseString: string; Positions: Integer): string;
var J: Integer;
    ResStr: string;
begin
  ResStr := '';
  for J := 1 to Positions-Length(BaseString) do ResStr := ResStr + ' ';
  Result := ResStr + BaseString;
end;

function NormalizeDir(DirStr: string): string;
begin
  if (Length(DirStr) > 0) and (DirStr[Length(DirStr)] <> '\') then Result := DirStr + '\'
  else Result := DirStr;
end;

function UnNormalizeDir(DirStr: string): string;
begin
  if DirStr[Length(DirStr)] = '\' then Result := Copy(DirStr, 1, Length(DirStr) - 1)
  else Result := DirStr;
end;

function BoolToStr(Value: Boolean): string;
begin
  if Value then Result := 'Yes' else Result := 'No';
end;

function StrToBool(Value: string): Boolean;
begin
  if Value = 'Yes' then Result := True else Result := False;
end;

function RealToStr(RealNumber: Real): string;
var S: string;
begin
  Str(RealNumber: 1: 2, S);
  Result := Trim(S);
end;

function StrToReal(RealStr: string): Real;
var I, DevPos: Integer;
    StrD: string;
    H: Integer;
    RealD: Real;
begin
  if Length(RealStr) = 0 then RealStr := '0';
  DevPos := 0;
  if Pos(',', RealStr) > 0 then DevPos := Pos(',', RealStr)
  else if Pos('.', RealStr) > 0 then DevPos := Pos('.', RealStr);
  if DevPos = 0 then Result := StrToInt(RealStr)
  else begin
    H := StrToInt(Trim(Copy(RealStr, 1, DevPos - 1)));
    StrD := Copy(RealStr, DevPos + 1, Length(RealStr) - DevPos);
    I := 0;
    while StrD[I] = '0' do begin
      StrD := Copy(StrD, 2, Length(StrD) - 1);
      Inc(I);
    end;
    if Length(StrD) > 0 then RealD := StrToInt(Trim(StrD))
    else RealD := 0;
    for I := 0 to Length(StrD) - 1 do RealD := RealD / 10;
    Result := H + RealD;
  end;
end;

procedure StringToFont(FontStr: string; Font: TFont);
var p: integer;
  sStyle: string;
begin
  with Font do begin
    p := Pos(',', FontStr);
    Name := Copy(FontStr, 2, p - 3);
    Delete(FontStr, 1, p);
    p := Pos(',', FontStr);
    Size := StrToInt(Copy(FontStr, 2, p - 2));
    Delete(FontStr, 1, p);
    p := Pos(',', FontStr);
    sStyle := '|' + Copy(FontStr, 3, p - 4);
    Delete(FontStr, 1, p);
    Color := StringToColor(Copy(FontStr, 3, Length(FontStr) - 3));
    Style := [];
    if (Pos('|Bold', sStyle) > 0) then Style := Style + [fsBold];
    if (Pos('|Italic', sStyle) > 0) then Style := Style + [fsItalic];
    if (Pos('|Underline', sStyle) > 0) then Style := Style + [fsUnderline];
    if (Pos('|Strikeout', sStyle) > 0) then Style := Style + [fsStrikeout];
  end;
end;

function FontToString(Font: TFont): string;
var sStyle: string;
begin
  with Font do begin
    sStyle := '';
    if (fsBold in Style) then sStyle := sStyle + '|Bold';
    if (fsItalic in Style) then sStyle := sStyle + '|Italic';
    if (fsUnderline in Style) then sStyle := sStyle + '|Underline';
    if (fsStrikeout in Style) then sStyle := sStyle + '|Strikeout';
    if ((Length(sStyle) > 0) and ('|' = sStyle[1])) then
      sStyle := Copy(sStyle, 2, Length(sStyle) - 1);
    Result := Format('"%s", %d, [%s], [%s]', [Name, Size, sStyle, ColorToString(Color)]);
  end;
end;

procedure RemoveDoubledStrings(AStrings: TStrings);
var StubStrings: TStrings;
  I: Integer;
begin
  StubStrings := TStringList.Create;
  for I := 0 to AStrings.Count - 1 do
    if StubStrings.IndexOf(AStrings[I]) = -1 then StubStrings.Add(AStrings[I]);
  AStrings.Assign(StubStrings);
  StubStrings.Free;
end;

function ColorToHexStr(AColor: TColor): string;
begin
  Result := IntToHex(ColorToRGB(AColor), 6);
  Result := Copy(Result, 5, 2) + Copy(Result, 3, 2) + Copy(Result, 1, 2);
end;

function AddDelString(AStrings: TStrings; StrToAddDel: string): Integer;
begin
  Result := AStrings.IndexOf(StrToAddDel);
  if StrToAddDel = '' then Exit;
  if Result = -1 then begin
    AStrings.Add(StrToAddDel);
    Result := AStrings.IndexOf(StrToAddDel);
  end
  else begin
    AStrings.Delete(Result);
    if AStrings.Count < 1 then Exit;
    if Result > AStrings.Count - 1 then Result := Result - 1;
  end;
end;

function LeadingZero(IntNumber, TotalCount: Integer): string;
var S: string;
begin
  S := IntToStr(IntNumber);
  if Length(S) > TotalCount then Result := S
  else begin
    while Length(S) < TotalCount do S := '0' + S;
    Result := S;
  end;
end;

function MatchStrings(source, pattern: String): Boolean;
var
  pSource: Array [0..255] of Char;
  pPattern: Array [0..255] of Char;

  function MatchPattern(element, pattern: PChar): Boolean;

    function IsPatternWild(pattern: PChar): Boolean;
    begin
      Result := StrScan(pattern,'*') <> nil;
      if not Result then Result := StrScan(pattern,'?') <> nil;
    end;

  begin 
    if 0 = StrComp(pattern,'*') then 
      Result := True 
    else if (element^ = Chr(0)) and (pattern^ <> Chr(0)) then 
      Result := False 
    else if element^ = Chr(0) then 
      Result := True 
    else begin 
      case pattern^ of 
      '*': if MatchPattern(element,@pattern[1]) then 
             Result := True 
           else 
             Result := MatchPattern(@element[1],pattern); 
      '?': Result := MatchPattern(@element[1],@pattern[1]); 
      else 
        if element^ = pattern^ then 
          Result := MatchPattern(@element[1],@pattern[1]) 
        else 
          Result := False; 
      end; 
    end; 
  end;

begin 
  StrPCopy(pSource,source); 
  StrPCopy(pPattern,pattern); 
  Result := MatchPattern(pSource,pPattern); 
end; 



end.


